document.getStreams = function () {
    return ['electrics', 'forcedInductionInfo'];
};

var width = 660;
var height = 660;
var initialized = false;
var pressure_redline_len = 0;
var dashSize = 7;

// pressure
var pressureText = document.getElementById('pressureText');
pressureText.textContent = '';

// pressureCurve dashes
var pressureCurveDashes = document.getElementById('pressureCurve_dashes');
var pressureCurveDashesLen = pressureCurveDashes.getTotalLength();

// redline
var redLine = document.getElementById('pressure_redline');
var redLineLen = redLine.getTotalLength();

// pressureTextGuideLine -- a helping line to position the text
var pressureTextGuideLine = document.getElementById('pressuretextline');
var pressureTextGuideLineLen = pressureTextGuideLine.getTotalLength();
pressureTextGuideLine.style['display'] = 'none';

// the pressure text
var pressureTextSize = 50; // in px
var dashCount = 14;

// setup + hide all
for (var k = 1; k < dashCount / 2 + 1; k++) {
    document.getElementById('pressuretext' + k).style['visibility'] = 'hidden';
    document.getElementById('pressuretext' + k).style['font-size'] = pressureTextSize + 'px';
}
var PRESURE_MAX_CONST = 150;
var PRESURE_MIN_CONST = -100;
var pressure_max;
var pressure_min;
//dashCount = 3; // this can artifically limit the number of rpm splits

// pressure_needle
var pressure_needle = document.getElementById('pressure_needle_d');
var pressureunitText = document.getElementById('pressureunit');

// Lights
var forcedInductionGlow = document.getElementsByClassName("fi_glow");

var displayMode = 2;

function applyData(data) {
    var boost = UnitPressure(data.boost);

    pressureText.textContent = document.roundDec(boost, 1);

    if (typeof data.lowBeam !== 'undefined' && typeof data.highBeam !== 'undefined') {
        var lightsOff = true;
        var lb = (data.lowBeam > 0.9);
        var hb = (data.highBeam > 0.9);
        if (lb) { lightsOff = false; }
        if (hb) { lightsOff = false; }
        
        for (_i = 0; _i < forcedInductionGlow.length; _i++) {
            forcedInductionGlow[_i].style['display'] = lightsOff ? 'none' : 'inline';
        }
    } else {
        for (_i = 0; _i < forcedInductionGlow.length; _i++) {
            forcedInductionGlow[_i].style['display'] = 'none';
        }
    }
    // (boost % of scala) / (displayed range)
    var percPos = (boost - pressure_min) / (pressure_max - pressure_min);

    // just in case the boost has inconsistent data
    if (percPos < 0) {
        percPos = 0;
    }
    if (percPos > 1) {
        percPos = 1;
    }

    // % of scala * scala deg
    // orig transform is vertical so -135 to "move" 0 to start of curve
    pressure_needle.setAttribute('transform', 'rotate(' + (percPos * 270 - 135) + ',' + (width / 2) + ',' + (height / 2) + ')');
}

var data = {};

document.reset = function () {
    initialized = false;
    for (var k = 1; k < dashCount / 2 + 1; k++) {
        document.getElementById('pressuretext' + k).style['visibility'] = 'hidden';
    }
}

document.isStreamValid = function (streams) { return !!streams.forcedInductionInfo; };

document.update = function (streams) {
    var data = streams.forcedInductionInfo;
    // real data mode, no animation

    pressure_max = Math.ceil(UnitPressure(Math.max(PRESURE_MAX_CONST, data.maxBoost)));
    pressure_min = Math.floor(UnitPressure(PRESURE_MIN_CONST));

    if (!initialized) {

        document.reset();

        var redline_perc = 0.8; //TODO: should make this calculated with engine damage that can make the turbo

        redLine.style['stroke-dasharray'] = redLineLen + ' ' + redLineLen;
        redLine.style['stroke-dashoffset'] = -redline_perc * redLineLen;

        dashSize = 14;
        dash2Size = 7;
        var dashes = (pressureCurveDashesLen - (dashSize * (dashCount) * 1.0 / 2 + dash2Size * (dashCount - 2) * 1.0 / 2)) * 1.0 / (dashCount - 2);
        pressureCurveDashes.style['stroke-dasharray'] = dashSize + ' ' + dashes + ' ' + dash2Size + ' ' + dashes;

        // setup and move the pressure gauge text
        var n = dashCount / 2;
        var lastT = 0;
        for (var k = 0; k < n; k++) {
            var pos = pressureTextGuideLine.getPointAtLength(pressureTextGuideLineLen * (k / (n - 1)));
            var rp = document.getElementById('pressuretext' + (k + 1) + 'p');
            var cx = rp.getAttribute('x');
            var cy = rp.getAttribute('y');
            rp.setAttribute('transform', 'translate(' + (pos.x - cx) + ',' + (pos.y - cy) + ')');
            document.getElementById('pressuretext' + (k + 1)).textContent = document.roundDec(k * ((pressure_max - pressure_min) / (n - 1)) + pressure_min, (pressure_max > 10 ? 0 : 1));
            document.getElementById('pressuretext' + (k + 1)).style['visibility'] = 'visible';
        }
        initialized = true;
    }

    // if(streams.forcedInductionInfo.boost < 0 &amp;&amp; unitSystem != "metric")
    // {
    //   //psi to in.HG when pressure is vaccum
    //   data.psi = (streams.forcedInductionInfo.boost * 2.03) / (30 * 2);
    //   data.pressureText = streams.forcedInductionInfo.boost * 2.03;
    // }
    // else
    // {
    //   data.pressureText = streams.forcedInductionInfo.boost * pressureunitMultiplier;
    //   data.psi = streams.forcedInductionInfo.boost / pressure_max;
    // }

    data.lowBeam = streams.electrics.lowbeam;
    data.highBeam = streams.electrics.highbeam;

    applyData(data);
}

var UiUnitscallback;

function UnitPressure(val) {
    var convertedVal = UiUnitscallback(val, 'pressure');

    if (pressureunitText.textContent !== convertedVal.unit) {
        pressureunitText.textContent = convertedVal.unit;
        // the scala needs to be changed
        initialized = false;
    }

    return convertedVal.val;
}

document.roundDec = () => 0;

document.wireThroughUnitSystem = function (callback) {
    UiUnitscallback = callback;
};